%% Define one of the three accelerometers 
chosenAccelerometer = accelerometerX;

%% BANDPASS FILTER 
f1=0.5; % cuttoff low frequency to get rid of baseline wander
f2=6; % cuttoff frequency to discard high frequency noise
Wn=[f1 f2]*2/30; % cutt off based on fs
N = 3; % order of 3 less processing
[a,b] = butter(N,Wn); % bandpass filtering
highPass = filtfilt(a,b,chosenAccelerometer);
highPass = highPass/ max( abs(highPass));

%% DERIVATIVE
%Two-point differentiation 
derivative = diff(highPass);
derivative = [derivative; zeros(1,1)];
derivative = derivative / max(abs(derivative));

%% SQUARING FUNCTION
squaring = derivative.^2;

% Find signal peaks - peaks under a threshold value are considered as noise.
[squaringPks, squaringPeakLocation] = findpeaks(squaring, 'MINPEAKHEIGHT', 0.1);

if isempty(squaringPks) 
    text = 'Squaring: No detected peaks'
else 
    numberOfStepsSquaring = length(squaringPks);

    % Find the time at which a step occur
    squaringPlotPointsX = zeros(1,length(squaringPks)); 
    for i=1:numberOfStepsSquaring
        squaringPlotPointsX(i) = second(squaringPeakLocation(i));
    end
end

%% MOVING-WINDOW INTEGRATION 
N = 5; 
movingWindow = conv(squaring,ones(1,N)/N);
movingWindow(numberOfsamples+1:end) = []; 

%Find signal peaks - peaks under a threshold value are considered as noise.
[movingWindowPks, mwPeakLocation] = findpeaks(movingWindow, 'MINPEAKHEIGHT', 0.1);

if isempty(movingWindowPks) 
    text = 'MovingWindow: No detected peaks'
else 
    numberOfStepsMovingWindow=length(movingWindowPks);

    % Find the time at which a step occur
    movingWindowPlotPointsX = zeros(1,length(movingWindowPks)); 
    for i=1:numberOfStepsMovingWindow
        movingWindowPlotPointsX(i) = second(mwPeakLocation(i));
    end

    % Save number of steps
    numberOfStepsSqrTable(iFile) = numberOfStepsSquaring; 
    numberOfStepsMWTable(iFile) = numberOfStepsMovingWindow;
end
