%% Calculate Magnitude of Triaxial Accelrometer
magnitude = sqrt(accelerometerX.*accelerometerX + accelerometerY.*accelerometerY + accelerometerZ.*accelerometerZ);
magnitude = magnitude - magnitude(1); % starting at zero

%figure(41)
%plot(second, magnitude);
%title('Magnitude of 3D Accelerometer Vector');

%% Find signal peaks in the original signal 
% - peaks under a threshold value are considered as noise.
thresholdValue = 0.1;
[magnitudePks,peakLocation] = findpeaks(magnitude, 'MINPEAKHEIGHT', thresholdValue);

% Calculate the number of steps based on the Median Filter, asuming a step has occured at every peak.  
if isempty(magnitudePks) 
    text = 'Magnitude: No detected peaks'
    numberOfStepsMag = 0; 
else
    numberOfStepsMag = length(magnitudePks) 
    % Find the time at which a step occur
    timeOfStepsMag = zeros(length(magnitudePks),1); 
    for i=1:numberOfStepsMag
        timeOfStepsMag(i) = second(peakLocation(i));
    end
    
    % Save number of steps
    numberOfStepsMagTable(iFile) = numberOfStepsMag;
    
    %figure(42)
    %plot(second, magnitude, timeOfStepsMag, magnitudePks, 'o');
    %title('Detected Peaks in Magnitude Signal');
    %legend({'Magnitude Signal' 'Located Local Max'}, 'Location','NorthEast')
end

%% DERIVATIVE
%Two-point difference 
derivative = diff(magnitude); %Difference and approximate derivative
derivative = [derivative; zeros(1,1)];  % Add zero to the last element in "derivative"
derivative = derivative / max(abs(derivative)); % Normalize to one 

%% Zero-crossing
% Find signal peaks. 
% - peaks under a threshold value are considered as noise.
derivativePks = findpeaks(derivative, 'MINPEAKHEIGHT', thresholdValue); 

numberOfStepsDer = 0;

if isempty(derivativePks)
    text = 'Differentiator: No detected peaks'
else
    timeOfStepsDer = zeros(1, numberOfsamples);
    zeroCrossing = zeros(1,numberOfsamples); 
    
    % Count the number of steps, where a step is defines as a zero-crossing from negative to positive:
    signalLessThanZero = false; 
    lessThanZeroPoint = 0;
    peakCounter = 1;
    plottingCounter = 1;

    for i=1:numberOfsamples
        
        % Find min peak, before a max peak
        if derivative(i) < -thresholdValue
            signalLessThanZero=true; 
            lessThanZeroPoint = i;
        end

        if derivative(i) == derivativePks(peakCounter) 

            % Adjust pekCounter
            if(peakCounter==length(derivativePks))
                peakCounter = length(derivativePks);
            else
                peakCounter = peakCounter+1;
            end

            %Peak after crossing zero from negative to positive
            if signalLessThanZero == true
                signalLessThanZero = false; 
                numberOfStepsDer = numberOfStepsDer +1;

                % Find time stamp of zero-crossing, using interpolation 
                samples = i - lessThanZeroPoint+1;

                xi = zeros(1,samples);
                yi = zeros(1, samples);
                counter = 1; 

                for k=1:samples
                    if k==1 && derivative(k+lessThanZeroPoint-1)>=derivative(k+lessThanZeroPoint)
                        %ikke lagre
                    elseif k==samples && derivative(k+lessThanZeroPoint-1) <= derivative(k+lessThanZeroPoint-2)
                        % ikke lagre 
                    elseif ne(k,1) && ne(k,samples) && (ge(derivative(k+lessThanZeroPoint-1),derivative(k+lessThanZeroPoint)) || le(derivative(k+lessThanZeroPoint-1),derivative(k+lessThanZeroPoint-2))) 
                        % ikke lagre 
                    else 
                        xi(counter) = second(k+lessThanZeroPoint-1);
                        yi(counter) = derivative(k+lessThanZeroPoint-1); 
                        counter = counter +1; 
                    end
                end
                xi(counter:end) = [];
                yi(counter:end) = [];

                zeroCrossingX = interp1(yi,xi, 0);
                zeroCrossingY = interp1(xi,yi, zeroCrossingX);
                timeOfStepsDer(plottingCounter) = zeroCrossingX;
                zeroCrossing(plottingCounter) = zeroCrossingY;

                plottingCounter = plottingCounter + 1; 
            end   % if peak after zero-crossing 
        end % if peak       
    end %runs through entire signal
    
    % Save number of steps
    numberOfStepsDerTable(iFile) = numberOfStepsDer;
    
    % Delete unused elements of plotting variables
    timeOfStepsDer(plottingCounter:end) = []; 
    zeroCrossing(plottingCounter:end) = [];

    %figure(43)
    %plot(second, derivative, timeOfStepsDer, zeroCrossing, 'o');
    %legend({'Output of Two-Point Derivative' 'Zero-crossing from negative to positive'}, 'Location','NorthEast')
    %title('Output of Derivative Operator'); 
end 
